/*
 * vim:ts=8:sw=3:sts=8:noexpandtab:cino=>5n-3f0^-2{2
 */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <fnmatch.h>
#include <Evas.h>
#include <Ecore.h>
#include <Ecore_File.h>
#include <Ecore_Str.h>
#include <Ecore_Evas.h>
#include <Edje.h>
#include "enna.h"
#include "enna_module.h"
#include "enna_list.h"
#include "enna_config.h"
#include "enna_mediaplayer.h"

#define SMART_NAME "enna_fm"
#define API_ENTRY E_Smart_Data *sd; sd = evas_object_smart_data_get(obj); if ((!obj) || (!sd) || (evas_object_type_get(obj) && strcmp(evas_object_type_get(obj), SMART_NAME)))
#define INTERNAL_ENTRY E_Smart_Data *sd; sd = evas_object_smart_data_get(obj); if (!sd) return;
typedef struct _E_Smart_Data E_Smart_Data;

struct _E_Smart_Data
  {
    Evas_Coord   	x, y, w, h;
    Evas_Object   	*edje;
    Evas_Object         *obj;
    Evas_Object	        *switcher;
    Evas_Object	        *o_files;
    char 		*root;
    char		*path;
    char		*filter;
    void               (*exe_file_cb) (void *data1, void *data2, char *filename);
    void                *data1;
    void                *data2;
    void               (*hilight_file_cb) (void *data1, void *data2, char *filename);
    void                *hilight_data1;
    void                *hilight_data2;
    void               (*exit_cb) (void *data1, void *data2);
    void                *exit_data1;
    void                *exit_data2;
  };

/* local subsystem functions */

static void _switch_objects(E_Smart_Data *sd, Evas_Object *obj1, Evas_Object *obj2);
static void _scan_dir(E_Smart_Data *sd, char *path, char *oldpath);
static int _late_clear (void *data);
static void _parent_go	(E_Smart_Data *sd, char *path);
static void _add_to_pl_cb	(E_Smart_Data *sd);
static void _add_to_pl(E_Smart_Data *sd, char *file);
static void _e_smart_reconfigure(E_Smart_Data *sd);
static void _e_smart_add(Evas_Object *obj);
static void _e_smart_del(Evas_Object *obj);
static void _e_smart_move(Evas_Object *obj, Evas_Coord x, Evas_Coord y);
static void _e_smart_resize(Evas_Object *obj, Evas_Coord w, Evas_Coord h);
static void _e_smart_show(Evas_Object *obj);
static void _e_smart_hide(Evas_Object *obj);
static void _e_smart_color_set(Evas_Object *obj, int r, int g, int b, int a);
static void _e_smart_clip_set(Evas_Object *obj, Evas_Object * clip);
static void _e_smart_clip_unset(Evas_Object *obj);
static void _e_smart_init(void);

/* local subsystem globals */
static Evas_Smart *_e_smart = NULL;

/* externally accessible functions */
EAPI Evas_Object *
enna_fm_add(Evas *evas)
{
  _e_smart_init();
  return evas_object_smart_add(evas, _e_smart);
}

EAPI void
enna_fm_process_event( Evas_Object *obj, enna_event event)
{
  API_ENTRY return;

  if (event == enna_event_escape)
    {
      //dbg("enna_fm: go_parent ev %d %s\n", event, sd->path);
       _parent_go(sd, sd->path);
    }
  else if (event == enna_event_addtopl)
    _add_to_pl_cb(sd);
  else
    enna_list_process_event(sd->o_files, event);
}

EAPI int
enna_fm_root_set(Evas_Object *obj, char* root_path)
{
  API_ENTRY return 0;
  if (!root_path || !ecore_file_is_dir(root_path))
    return 0;
  sd->root = strdup(root_path);
  if (sd->o_files)
    evas_object_del(sd->o_files);
  _scan_dir(sd, sd->root, NULL);
  return 1;
}

EAPI int
enna_fm_filter_set(Evas_Object *obj, char* filter)
{
  API_ENTRY return 0;
  if (!filter)
    return 0;
  sd->filter = strdup(filter);
  return 1;
}

EAPI int
enna_fm_exe_file_cb_set( Evas_Object *obj, void (exe_file_cb)(void *data1, void *data2, char *filename), void *data1, void *data2)
{
  API_ENTRY return 0;
  sd->exe_file_cb = exe_file_cb;
  sd->data1 = data1;
  sd->data2 = data2;
  return 1;
}

EAPI int
enna_fm_hilight_file_cb_set( Evas_Object *obj, void (hilight_file_cb)(void *data1, void *data2, char *filename), void *data1, void *data2)
{
  API_ENTRY return 0;
  sd->hilight_file_cb = hilight_file_cb;
  sd->hilight_data1 = data1;
  sd->hilight_data2 = data2;
  return 1;
}

EAPI int 
enna_fm_exit_cb_set( Evas_Object *obj, void (exit_cb)(void *data1, void *data2), void *data1, void *data2)
{
  API_ENTRY return 0;
  sd->exit_cb = exit_cb;
  sd->exit_data1 = data1;
  sd->exit_data2 = data2;
  return 1;
}

static int
_has_suffix(char* str, Evas_List* patterns)
{
  Evas_List *l;
  int result = 0;
  if (!patterns || !str)
    return 0;
  for (l = patterns; l; l = evas_list_next(l))
    result |= ecore_str_has_suffix(str, (char*)l->data);
  return result;
}

static void
_directory_selected_cb (void *data, void *data2)
{
  E_Smart_Data *sd;
  char		*path;
  sd = (E_Smart_Data*) data;
  path = (char*) data2;
  if (!sd || !path || !ecore_file_is_dir(path))
    return;
  _scan_dir(sd, path, NULL);
}

static void
_hilight_file_cb (void *data, void *data2)
{
  E_Smart_Data *sd;
  char		*path;

  sd = (E_Smart_Data*) data;
  path = (char*) data2;

  if (sd->hilight_file_cb)
    sd->hilight_file_cb(sd->hilight_data1, sd->hilight_data2, path);
 
}

static void
_file_selected_cb (void *data, void *data2)
{
  E_Smart_Data *sd;
  char		*path;
  sd = (E_Smart_Data*) data;
  path = (char*) data2;
  if (sd->exe_file_cb)
    sd->exe_file_cb(sd->data1, sd->data2, path);
}

static void
_switch_objects(E_Smart_Data *sd, Evas_Object *obj1, Evas_Object *obj2)
{
  if (!sd || (!obj1 && !obj2))
    return;

  if ( (obj1 && obj2) )
    {
      edje_object_part_unswallow(sd->switcher, obj1);
      edje_object_part_swallow(sd->switcher, "enna.switcher.swallow2", obj1);
      edje_object_signal_emit(sd->switcher, "enna,swallow2,default,now", "enna");
      edje_object_signal_emit(sd->switcher, "enna,swallow1,state1,now", "enna");
      edje_object_part_swallow(sd->switcher, "enna.switcher.swallow1", obj2);
      edje_object_signal_emit(sd->switcher, "enna,swallow1,default", "enna");
      edje_object_signal_emit(sd->switcher, "enna,swallow2,state2", "enna");
      evas_object_hide(obj1);
    }
  else if ( !obj2 )
    {
      edje_object_part_unswallow(sd->switcher, obj1);
      edje_object_part_swallow(sd->switcher, "enna.switcher.swallow2", obj1);
      edje_object_signal_emit(sd->switcher, "enna,swallow2,default,now", "enna");
      edje_object_signal_emit(sd->switcher, "enna,swallow2,state2", "enna");
    }
  else if ( !obj1 )
    {
      edje_object_signal_emit(sd->switcher, "enna,swallow1,state1,now", "enna");
      edje_object_part_swallow(sd->switcher, "enna.switcher.swallow1", obj2);
      edje_object_signal_emit(sd->switcher, "enna,swallow1,default", "enna");
      evas_object_show(obj2);
    }
}

static int
_late_clear (void *data)
{
  Evas_Object* obj;

  obj = (Evas_Object*) data;
  evas_object_hide(obj);
  evas_object_del(obj);

  return 0;
}

static void 
_scan_dir(E_Smart_Data *sd, char *path, char *oldpath)
{

  Ecore_List *files = NULL;
  char *filename = NULL;
  char dir[1024];
  Evas_Object *o;
  int to_select = 0;
  int count = 0;


  if ( !sd || !path || !ecore_file_is_dir(path))
    return;

  files = ecore_file_ls (path);
  filename = ecore_list_goto_first (files);
  o = sd->o_files;
  sd->o_files = enna_list_add(evas_object_evas_get(sd->edje));
  sd->path = strdup(path);
  evas_object_show(sd->o_files);
  while ((filename = (char *) ecore_list_next (files)) != NULL)
    {
      sprintf (dir, "%s/%s", path, filename);
      if (filename[0] == '.')
        continue;
      else if (ecore_file_is_dir (dir))
        {
          Evas_Object *icon;
          char *file;
          
          icon = edje_object_add(evas_object_evas_get(sd->edje));
          edje_object_file_set(icon, enna_config_theme_get(), "icon_directory");
          evas_object_show(icon);
          file = strdup(dir);
	  enna_list_append(sd->o_files, icon,  filename , 0, _directory_selected_cb, NULL, sd, file);
	  if (oldpath)
	    if (!strcmp(dir, oldpath))
	      to_select = count;
	  count++;
	}
      else if (_has_suffix(filename, enna_config_extensions_get(sd->filter)))
	{
	   Evas_Object *icon;
	   char *file;
	   char tmp[256];
	   icon = edje_object_add(evas_object_evas_get(sd->edje));
	   sprintf(tmp, "icon_%s", sd->filter);
	   edje_object_file_set(icon, enna_config_theme_get(), tmp);
	   evas_object_show(icon);
	   file = strdup(dir);
	   enna_list_append(sd->o_files, icon,  filename , 0, _file_selected_cb, _hilight_file_cb, sd,  file);
	   count++;
	}
    }
  _switch_objects(sd, o, sd->o_files);
  ecore_timer_add (1.0, _late_clear, o);
  enna_list_selected_set(sd->o_files, to_select);
}

static void
_parent_go(E_Smart_Data *sd, char *path)
{
  char  *p;
  char *path_tmp;
  if (!sd || !path || !sd->root)
    return;

  if (!strcmp(path, sd->root) || !path)
    {
      if (sd->exit_cb)
        sd->exit_cb(sd->exit_data1, sd->exit_data2);
      return;
    }
  path_tmp = strdup(path);

  if (path_tmp[strlen(path_tmp)-1] == '/' )
    path_tmp[strlen(path_tmp)-1] = 0;
  p = strrchr(path_tmp, '/');
  if (p && *(p-1) == '/')
    *(p) = 0;
  else if (p)
    *(p) = 0;
  _scan_dir( sd, path_tmp, path);
  // Fixme: memory leak
}

static void 
_add_to_pl(E_Smart_Data *sd, char *file)
{
  Enna *enna;
  Ecore_List *files = NULL;
  char *filename;
  
  enna = evas_data_attach_get(evas_object_evas_get(sd->edje));
  if (!sd || !file || !enna)
    return;

  if (!ecore_file_is_dir(file) && _has_suffix(file, enna_config_extensions_get(sd->filter)))
  {
    dbg(" ---> Append  --  %s  -- to playlist\n", file);
    enna_mediaplayer_playlist_add(enna->mediaplayer, file);
  }
  else if (ecore_file_is_dir(file))
  {
    enna_mediaplayer_playlist_add(enna->mediaplayer, file);
    files = ecore_file_ls (file);
    filename = ecore_list_goto_first (files);
    while ((filename = (char *) ecore_list_next (files)) != NULL)
    {
      char tmp[4096];
      sprintf (tmp, "%s/%s", file, filename);
      _add_to_pl(sd, tmp);
    }
  }
}

static void 
_add_to_pl_cb	(E_Smart_Data *sd)
{
  char* file;
  
  file = (char*)enna_list_selected_data2_get(sd->o_files);
  
  enna_list_in_playlist_set(sd->o_files, 1, enna_list_selected_get(sd->o_files));
  
  _add_to_pl(sd, file);
}

static void
_e_smart_reconfigure(E_Smart_Data *sd)
{
  evas_object_move(sd->edje, sd->x, sd->y);
  evas_object_resize(sd->edje, sd->w, sd->h);
}

static void
_e_smart_add(Evas_Object *obj)
{
  E_Smart_Data *sd;
  sd = calloc(1, sizeof(E_Smart_Data));
  if (!sd)
    return;
  evas_object_smart_data_set(obj, sd);

  sd->obj = obj;
  sd->x = 0;
  sd->y = 0;
  sd->w = 0;
  sd->h = 0;

  sd->edje = edje_object_add(evas_object_evas_get(obj));
  edje_object_file_set(sd->edje, enna_config_theme_get(), "enna/filemanager");
  evas_object_smart_member_add(sd->edje, obj);
  evas_object_show(sd->edje);

  sd->switcher = edje_object_add(evas_object_evas_get(obj));
  edje_object_file_set(sd->switcher, enna_config_theme_get(), "enna/switcher");
  evas_object_smart_member_add(sd->switcher, obj);
  evas_object_show(sd->switcher);
  edje_object_part_swallow(sd->edje, "enna.switcher.swallow", sd->switcher);
  sd->o_files = NULL;
  sd->root= NULL;
  sd->path = NULL;
  sd->filter = NULL;
  sd->exe_file_cb = NULL;
  sd->data1 = NULL;
  sd->data2 = NULL;
  evas_object_propagate_events_set(obj, 0);
}

static void
_e_smart_del(Evas_Object *obj)
{
  INTERNAL_ENTRY;
  evas_object_del(sd->o_files);
  evas_object_del(sd->switcher);
  evas_object_del(sd->edje);
  ENNA_FREE(sd->root);
  ENNA_FREE(sd->path);
  ENNA_FREE(sd->filter);
  ENNA_FREE(sd);
}

static void
_e_smart_move(Evas_Object *obj, Evas_Coord x, Evas_Coord y)
{
  INTERNAL_ENTRY;
  if ((sd->x == x) && (sd->y == y))
    return;
  sd->x = x;
  sd->y = y;
  _e_smart_reconfigure(sd);
}

static void
_e_smart_resize(Evas_Object *obj, Evas_Coord w, Evas_Coord h)
{
  INTERNAL_ENTRY;

  if ((sd->w == w) && (sd->h == h))
    return;
  sd->w = w;
  sd->h = h;
  _e_smart_reconfigure(sd);

}

static void
_e_smart_show(Evas_Object *obj)
{
  INTERNAL_ENTRY;
  evas_object_show(sd->edje);
}

static void
_e_smart_hide(Evas_Object *obj)
{
  INTERNAL_ENTRY;
  evas_object_hide(sd->edje);
}

static void
_e_smart_color_set(Evas_Object *obj, int r, int g, int b, int a)
{
  INTERNAL_ENTRY;
  evas_object_color_set(sd->edje, r, g, b, a);
}

static void
_e_smart_clip_set(Evas_Object *obj, Evas_Object * clip)
{
  INTERNAL_ENTRY;
  evas_object_clip_set(sd->edje, clip);
}

static void
_e_smart_clip_unset(Evas_Object *obj)
{
  INTERNAL_ENTRY;
  evas_object_clip_unset(sd->edje);
}

/* never need to touch this */

static void
_e_smart_init(void)
{
  if (_e_smart)
    return;
  _e_smart = evas_smart_new(SMART_NAME,
                            _e_smart_add,
                            _e_smart_del,
                            NULL, NULL, NULL, NULL, NULL,
                            _e_smart_move,
                            _e_smart_resize,
                            _e_smart_show,
                            _e_smart_hide,
                            _e_smart_color_set,
                            _e_smart_clip_set,
                            _e_smart_clip_unset,
                            NULL);
}
